#include <stdio.h>
#include <string.h>
#include "gpmi.h"

/* This function is used to print a detailed error message */
void print_gpmi_error(gpmi_err_stack_t *entry, char *string)
{
	printf("-> %s\n", string);
}

int main(int argc, char *argv[]) {
	gpmi_module *mod_script;
	int n;

	if (argc < 3) {
		printf("Usage: %s module script\n", argv[0]);
		printf("Example: %s tcl example_tcl\n", argv[0]);
		return 1;
	}

	gpmi_path_append(GPMI_PATH_MODULES, GPMI_ROOT "/modules");
	gpmi_path_append(GPMI_PATH_PACKAGES, GPMI_ROOT "/packages");
	gpmi_path_append(GPMI_PATH_SCRIPTS, GPMI_ROOT "/scripts");

	gpmi_init();
	gpmi_threads_start();

	gpmi_err_stack_enable();
	mod_script = gpmi_mod_load(argv[1], argv[2]);

	if (mod_script == NULL) {
		printf("Couldn't open %s module, stack:\n", argv[1]);
		gpmi_err_stack_process_str(print_gpmi_error);
		gpmi_uninit();
		return 1;
	}

	for(n = 0; n < 256; n++)
		gpmi_call_RunTick(mod_script, n);

	gpmi_call_RunData(mod_script, (void *)0x1234);

	gpmi_mod_unload(mod_script);

	gpmi_uninit();

	printf("Example finished.\n");

	return 0;
}
