#include <gpmi.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "gpmi/debug.h"
#include <ghli/ghli_helper.h>
#include "ghli_mod.h"


/**
 * Returns the extension of this script-module.
 */
char *GHLI_GetExtension()
{
	return strdup(".pas");
}

/**
 * Reloads the script on the fly.
 */
int GHLI_Rehash(void *client_data)
{
	/* XXX: not supported yet */
	return -1;
}

/**
 * Loads a ghli script.
 */
int GHLI_LoadScript(void *client_data, const char *filename)
{
	int ret;

	GHLI_ClientData *GHLI_MyData = client_data;

	ghli_load_script_stage2(GHLI_MyData->h, (char *)filename);
	ret = ghli_run(GHLI_MyData->h, -1);

	if (ret < 0)
		GHLI_ReportGHLIError(client_data, ret);

	return 0;
}

/**
 * Reports a ghli error using DEBUG.
 */
void GHLI_ReportGHLIError(GHLI_ClientData *GHLI_MyData, int error)
{
	gpmi_error(0, "[GHLI] Ghli runtime error: %d\n", error);
}

/**
 * Execute a dynamic command.
 **/
int GHLI_DynamicPackageCommand(ghli_t *h, int par_num, ghli_typevar_t *param, ghli_ext_call_t *ex)
{
//	GHLI_ClientData *GHLI_MyData = GPMI_Script_GetClientData();
	char *(*proc)(int argc, char *argv[]);
	char *s;
	char **argv;
	int n;

	proc = GPMI_Script_FindProc(ex->name);

	/* Check if command not found - this must be an internal error: the
	   package had registered the command in the hash, but the routine
	   pointer is NULL.
	*/
	if (proc == NULL) {
		gpmi_error(0, "[GHLI] Internal error: %s has a NULL proc in the command hash\n", ex->name);
		return -1;
	}

	/* We convert each parameter to string and store them in an char *argv[] array */
	argv = malloc(sizeof(char *) * (par_num + 1));
	argv[0] = ex->name;
	for(n = 0; n < par_num; n++) {
		if (ghli_var_convert(&(param[n]), ghli_type_string) != 0) {
			gpmi_error(0, "[GHLI] Internal error: Can not convert %dth parameter to string for %s\n", n, ex->name);
			free(argv);
			return -1;
		}

		/* We do not strdup here, so we don't need to care about the string, ghli will free it */
		argv[n+1] = param[n].var.string;
	}

	/* Run command */
	s = proc(par_num + 1, argv);

	if (s != NULL) {
		/* Allow pointers to convert to string */
		if ((s[0] == '0') && (s[1] == 'x'))
			ghli_result_string(h, s);
		else
			ghli_result_auto(h, s);

		/* Free result */
		free(s);
	}
	else
		ghli_result_string(h, "");

	free(argv);
	return 0;
}

/**
 * Register a command in our command hash
 */
void GHLI_RegisterCommand(void *client_data, const char *name)
{
	GHLI_ClientData *GHLI_MyData = client_data;

	/* Use direct calling for ghli units and a common gateway for the rest */
	if (strncmp(name, "ghli_wrap_", 10) == 0)
		ghli_ext_bind(GHLI_MyData->h, (char *)(name + 10), gpmi_dlsym_lowlevel(GHLI_MyData->loading->library, (char *)name));
	else
		ghli_ext_bind(GHLI_MyData->h, (char *)name, GHLI_DynamicPackageCommand);
}

/**
 * Call a function of the script by name, passing params.
 */
void GHLI_RunFunction(void *client_data, const char *function, int argc, const char **argv)
{
	GHLI_ClientData *GHLI_MyData = client_data;
	int ret;

	ret = ghli_call_function_str(GHLI_MyData->h, (char *)function, argc, (char **)argv);
	if (ret != 0) {
		gpmi_error(0, "[GHLI] Runtime error %d while running '%s'\n", ret, function);
	}
}

int GHLI_Initialize(void **client_data)
{
	GHLI_ClientData *MyData;

	MyData = malloc(sizeof(GHLI_ClientData));

	MyData->h = ghli_load_script_stage1(1024, 128, 512);
	MyData->script_name = NULL;

	*client_data = MyData;

	return 0;
}

void GHLI_Destroy(void **client_data)
{
	GHLI_ClientData *GHLI_MyData = *client_data;

	/* Not initialized */
	if (GHLI_MyData->script_name == NULL)
		return;

	/* Free the script */
	free(GHLI_MyData->script_name);
	GHLI_MyData->script_name = NULL;

	ghli_cleanup(GHLI_MyData->h);

	/* Finally free up the client-data */
	free(*client_data);
}

void GHLI_SetPkg(void *client_data, const gpmi_package *package)
{
	GHLI_ClientData *MyData = client_data;

	MyData->loading = package;
}
